import 'package:ansicolor/ansicolor.dart';
import 'package:stack_trace/stack_trace.dart';

import '../models/classes.dart';
import '../models/enums.dart';
import '../utils/utils.dart';
import 'log_file.dart';

class LoggerImpl implements Logger {
  factory LoggerImpl(LoggerConfig loggerConfig) {
    LogFile logFile = LogFile.getInstance(loggerConfig);
    return LoggerImpl._(loggerConfig, logFile);
  }

  LoggerImpl._(this._config, this._logFile);

  final LoggerConfig _config;
  final LogFile _logFile;

  @override
  trace(dynamic message) {
    LoggerLevel level = LoggerLevel.trace;
    _log(message, level);
  }

  @override
  debug(dynamic message) {
    LoggerLevel level = LoggerLevel.debug;
    _log(message, level);
  }

  @override
  info(dynamic message) {
    LoggerLevel level = LoggerLevel.info;
    _log(message, level);
  }

  @override
  warn(dynamic message) {
    LoggerLevel level = LoggerLevel.warn;
    _log(message, level);
  }

  @override
  error(dynamic message) {
    LoggerLevel level = LoggerLevel.error;
    _log(message, level);
  }

  @override
  fatal(dynamic message) {
    LoggerLevel level = LoggerLevel.fatal;
    _log(message, level);
  }

  _log(dynamic message, LoggerLevel level) {
    String prefix = _getPrefix(level);
    String printString;
    if (message.runtimeType == String) {
      printString = prefix + message;
    } else {
      // TODO : format it
      printString = prefix + message.toString();
    }

    if (_config.printLevel.index <= level.index) {
      AnsiPen ansiPen = LoggerUtils.getColor(level);
      print(ansiPen(printString));
    }
    if (_config.fileLevel.index <= level.index) {
      _logFile.writeLog(printString);
    }
    if (_config.serverUrl.isNotEmpty &&
        _config.serverLevel.index <= level.index) {
      // TODO : upload it
    }
  }

  String _getPrefix(LoggerLevel level) {
    String currentTime = DateTime.now().toUtc().toString();
    String levelNmae = LoggerUtils.enumToString(level);
    String member = Trace.current().frames[4]?.member;
    return "$currentTime $levelNmae [$member] : ";
  }
}
